/*-------------------------------------------------------------------------
| File: epsbb.c [ANSI-C conforming source code]
| Description: The epsbb program extracts the bounding box of files
|              containing Encapsulated PostScript (EPS) images [1].
|                 The output is a string of coordinates of the EPS bounding
|              box, in the format "llx,lly,urx,ury", where (llx,lly) are
|              the lower left coordinate and (urx,ury) the upper right
|              coordinate, both given in pt (1/72 inch). The program
|              is essentially just a self-consistent driver for the
|              scan_for_boundingbox() routine that I wrote some years ago,
|              and this is definitively one of the most compact and simple
|              programs I have ever written. Nevertheless, I find it very
|              useful in scripts and Makefiles, where I occasionally want
|              to pipe the bounding box coordinate to other programs (such
|              as MetaPost or TeX).
|                 Notice that I have added the -w option, compared to the
|              classical version of epsbb, in order to get a more readable
|              output of the bounding box size in terms of millimetres.
|              This option is nice to use when invoking epsbb from the command
|              line (in order to conform to the classical behaviour, I have
|              though kept the default output coordinates in pt whenever the
|              program is invoked without any options).
|
| Created:     July 16, 2003
| Last change: August 5, 2004 [Adding the -v option.]
| Author: Fredrik Jonsson <hakkasberra@hotmail.com>
|
| Copyright (C) Fredrik Jonsson <hakkasberra@hotmail.com>
|
| Compile the source code using the following Makefile:
|     ----------------------- CUT HERE -----------------------
|     CC     = gcc
|     CCOPTS = -O2 -Wall -ansi -pedantic
|
|     epsbb: epsbb.o
|             $(CC) $(CCOPTS) -o epsbb epsbb.o
|
|     epsbb.o: epsbb.c
|             $(CC) $(CCOPTS) -c epsbb.c
|
|     clean:
|             -rm -Rf *~ *.o *.exe
|     ----------------------- CUT HERE -----------------------
|
| [1] For information on the PostScript programming language,
|     see for example the homepage of Adobe Systems Inc., at
|     http://www.adobe.com/products/postscript/main.html,
|     or "PostScript Language - Tutorial and Cookbook"
|     (Addison-Wesley, Reading, Massachusetts, 1985), ISBN 0-201-10179-3.
-------------------------------------------------------------------------*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

extern char *optarg;
char *progname;

/*-------------------------------------------------------------------------
| The scan_for_boundingbox(infilename,llx,lly,urx,ury) routine takes the
| name of a regular ASCII text file (infilename) containing Encapsulated
| PostScript (EPS) as input, and returns the bounding box of the figure,
| in terms of the corner coordinates given by llx, lly, urx, and ury.
|
| Input parameters:
|    infilename  (type char[])  Filename of inut Encapsulated PostScript file
|                               including possible path specification, and
|                               not longer than 256 characters.
|
| Output parameters:
|    llx         (type *int)    Lower left x-coordinate of bounding box.
|    lly         (type *int)    Lower left y-coordinate of bounding box.
|    urx         (type *int)    Upper right x-coordinate of bounding box.
|    ury         (type *int)    Upper right y-coordinate of bounding box.
|
| Typical usage of this routine, in terms of a sample program for extraction
| of bounding box, similar to the UNIX standard program psbb, is:
|
|     #include <stdlib.h>
|     #include <stdio.h>
|     #include <string.h>
|
|     extern char *optarg;
|     char *progname;
|
|     void scan_for_boundingbox(char infilename[256],
|        long int *llx,long int *lly,long int *urx,long int *ury)
|     {
|        ...
|     }
|
|     int main(int argc,char *argv[])
|     {
|        char infilename[256];
|        int no_arg;
|        long int llx,lly,urx,ury;
|
|        progname=argv[0];
|        no_arg=argc;
|        if (argc<2) {
|           fprintf(stderr,"%s: Error! Please specify input (EPS) filename.\n",
|              progname);
|           fprintf(stderr,"Usage: %s <filename>\n",progname);
|           exit(1);
|        }
|        --argc;
|        strcpy(infilename,argv[no_arg-argc]);
|        scan_for_boundingbox(infilename,&llx,&lly,&urx,&ury);
|        fprintf(stdout,"%ld %ld %ld %ld\n",llx,lly,urx,ury);
|        exit(0);
|     }
|
-------------------------------------------------------------------------*/
void scan_for_boundingbox(char infilename[256],
   long int *llx,long int *lly,long int *urx,long int *ury)
{
   short int done;
   char tmpstr[256];
   int tmpch;
   FILE *infile;

   if ((infile=fopen(infilename,"r"))==NULL) {
      fprintf(stderr,"%s: Error! Could not open file %s for reading.\n",
         progname,infilename);
      exit(1);
   }
   fseek(infile,0L,SEEK_SET);
   done=0;
   while (((tmpch=getc(infile))!=EOF)&&(!done)) {
      ungetc(tmpch,infile);
      fscanf(infile,"%s",tmpstr);
      if (!strcmp(tmpstr,"%%BoundingBox:")) {
         done=1;
         fscanf(infile,"%ld",llx);
         fscanf(infile,"%ld",lly);
         fscanf(infile,"%ld",urx);
         fscanf(infile,"%ld",ury);
      }
   }
   if (tmpch==EOF) {
      fprintf(stderr,"%s: Error! End of file reached without ",progname);
      fprintf(stderr,"finding any %%%%BoundingBox statement!\n");
      fprintf(stderr,"%s: (Does %s really contain ",progname,infilename);
      fprintf(stderr,"Encapsulated PostScript?)\n");
      exit(1);
   }
}

void showsomehelp(void) {
   fprintf(stdout,"Usage: %s <filename> [options]\n",progname);
   fprintf(stdout,"The %s program extracts the bounding box of the ",progname);
   fprintf(stdout,"Encapsulated PostScript\nfile <filename>, and writes ");
   fprintf(stdout,"the bounding box corners to standard terminal\n");
   fprintf(stdout,"output in of the form <llx> <lly> <urx> <ury> and in ");
   fprintf(stdout,"units of typographical\npoints. [1 pt = 1/72.27 in]\n");
   fprintf(stdout,"Options:\n");
   fprintf(stdout,"  -v      Toggle verbose mode. (default: off)\n");
   fprintf(stdout,"  -w      Print height and width of bounding box in\n");
   fprintf(stdout,"          millimetres, instead of just printing the\n");
   fprintf(stdout,"          coordinates of the corners in points.\n");
   fprintf(stdout,"Copyright (C) 2002-2005, Fredrik Jonsson ");
   fprintf(stdout,"<fj@physics.soton.ac.uk>\n");
}

/*---------------------------------------------------------------------
| The pathcharacter() routine takes one character `ch` as argument,
| and returns 1 ('true') if the character is valid character of a path
| string, otherwise 0 ('false') is returned.
---------------------------------------------------------------------*/
short pathcharacter(int ch) {
   return(isalnum(ch)||(ch=='.')||(ch=='/')||(ch=='\\')||(ch=='_')||
      (ch=='-')||(ch=='+'));
}

/*---------------------------------------------------------------------
| The strip_away_path() routine takes a character string `filename` as
| argument, and returns a pointer to the same string but without any
| preceding path segments. This routine is, for example, useful for
| removing paths from program names parsed from the command line.
---------------------------------------------------------------------*/
char *strip_away_path(char filename[]) {
   int j,k=0;
   while (pathcharacter(filename[k])) k++;
   j=(--k); /* this is the uppermost index of the full path+file string */
   while (isalnum((int)(filename[j]))) j--;
   j++; /* this is the lowermost index of the stripped file name */
   return(&filename[j]);
}

int main(int argc,char *argv[])
{
   char infilename[256];
   int no_arg;
   long int llx,lly,urx,ury;
   short view_width_height=0,verbose=0;

   /* Parse command line for input parameters */
   progname=strip_away_path(argv[0]); /* Strip from preceeding path string */
   no_arg=argc;
   if (argc<2) {
      fprintf(stderr,"%s: Please specify input (EPS) filename.\n",
         progname);
      showsomehelp();
      exit(1);
   }
   --argc;
   strcpy(infilename,argv[no_arg-argc]);
   while (--argc) {
      if(!strcmp(argv[no_arg-argc],"-w")) {
         view_width_height=(view_width_height?0:1);
      } else if(!strcmp(argv[no_arg-argc],"-v")) {
         verbose=(verbose?0:1);
      } else {
         fprintf(stderr,"%s: Invalid option %s.\n",progname,argv[no_arg-argc]);
         showsomehelp();
         exit(1);
      }
   }

   /* Scan the specified Encapsulated PostScript file for its bounding box */
   scan_for_boundingbox(infilename,&llx,&lly,&urx,&ury);

   /* Write the bounding box to standard terminal output */
   if (verbose) {
      fprintf(stdout,"Bounding box of Encapsulated PostScript image %s:\n",
	      infilename);
   }
   if (view_width_height) {
      fprintf(stdout,"width=%-4.2f mm (%ld pt; %-4.2f in), ",
         (urx-llx)*(25.4/72.27),urx-llx,(urx-llx)*(1.0/72.27));
      fprintf(stdout,"height=%-4.2f mm (%ld pt; %-4.2f in)\n",
         (ury-lly)*(25.4/72.27),ury-lly,(ury-lly)*(1.0/72.27));
   } else {
      fprintf(stdout,"%ld %ld %ld %ld\n",llx,lly,urx,ury);
   }
   exit(0);
}
